"""
LightBrow: A Fast and Efficient S3 Browser and Search Engine
========================================================
"""

__version__ = '0.1.0'

from lightbrow.connectors import S3Connector
from lightbrow.browsers import S3Browser
from lightbrow.search_engines import SimpleSearch