"""
Dashboard module for BlazingPlot.

This module contains the Dashboard class which is the main entry point
for creating interactive dashboards with BlazingPlot.
"""

from lightbrow.search_engines.simple_seach import SimpleSearch
